/***************************************************************\
      This is All For Chinese Input/Output Method Proposal

  Author: Brian Lin
  Copyright: Open Source, GPL, Copylefted

\***************************************************************/

#ifndef __IOME_H
#define __IOME_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************\
  A4C IOME Configuration Proposal

  I strongly suggest IOME configuration should be unified.

  I recommend configuration file should be placed either
  in the /etc/iome/ directory 
  or 
  $HOME/.iome/ directory, 
  but not necessarily restrict.

  All For Chinese support multilingual configuration.
  
  system configuration file should be

  $HOME/.iome/configure   - System Configuration
   Keywords
     language definition - lang big5, gb2312
     configuration file  - configure iome.big5, iome.gb2312
     modules             - modules iome.modules
     display             - optional X DISPLAY NAME

  $HOME/.iome/conf.big5   - BIG5 Version Configuration
  $HOME/.iome/conf.gb2312 - GB Version Configuration

\*******************************************************/

typedef struct {
  char * key;
  char * value;
} iomeConfItem;

typedef struct {
  int            size;
  iomeConfItem * items;
} iomeConfig;

#define IOME_MAX_BUFF 2048

int iomeLoadConfiguration(char *FileName,iomeConfig *CONF);
int iomeFreeConfiguration(iomeConfig *CONF);

/* Utilities for Configuration */
char * iomeWhichItem(iomeConfig *CONF,char *key);

/* Default Configuration Loader */
typedef struct {
  iomeConfig SYS;
  iomeConfig CONF;
  iomeConfig MODULES;
} iomeDefaultConf;

int iomeDefaultLoader(iomeDefaultConf * DEFCONF);
int iomeTOUPPER(int ch);
int iomeTOLOWER(int ch);

/********************************************************\
                     Module functions
 --------------------------------------------------------
 Remember, modules are not allow to keep global variables
\********************************************************/
typedef char * iomeString;
typedef struct {
  char  * encoding;      /* big5, gb2312, ... */
  char  * class;         /* ime, filter */
  char ** accept_system; /* x,tty,null */
  char  * RESERVED[32];  /* Reserved */
} iomeRegistry;

typedef struct {
  char * system;
  char * platform;
  char * version;
  char * charset;
  char * RESERVED[32];  /* Reserved */
} iomeSysInfo;

typedef struct {
  char * MethodName; /* This name should be load from CONF */
  char * FilterName; /* This name should be load from CONF */
  char * Encoding;   /* big5, gb2312, ... */
  int    MaxListArea;
  char * ListArea;   /* Message for the ListArea, IME has to allocate itself, free it itself */
  int    MaxKeyArea;
  char * KeyArea;    /* Message for the KeyArea */
  char * Value;      /* Return Value */
  int    CWidth;     /* Font Size */
  int    CHeight;
  int    EWidth;
  int    EHeight;
  int    lwidth,lheight; /* ListArea Range */
  int    kwidth,kheight; /* KeyArea Range */  
  int    enable_cursor;
  int    cursor_x;
  int    RESERVED[62];
  void * Owner;      /* Owner's data */
} iomeRecord;

typedef int (*iomeEncoding)(unsigned char ch[]);
typedef int (*iomeRegister)(iomeSysInfo *INFO,iomeRegistry *REG);
typedef int (*iomeInitialize)(iomeConfig *CONF,iomeRecord *RECORD);
typedef int (*iomePrepare)(iomeRecord *RECORD);
typedef int (*iomeFeedKey)(iomeRecord *RECORD,int ch,int state);
typedef int (*iomeMouseOverListArea)(iomeRecord *RECORD,int x,int y);
typedef int (*iomeMouseOverKeyArea)(iomeRecord *RECORD,int x,int y);
typedef int (*iomeNeedFocus)(iomeRecord *RECORD);
typedef int (*iomeSimpleConvert)(iomeRecord *RECORD,char *charset,char *value);
typedef int (*iomeQuit)(iomeRecord *RECORD);

#define IME_REGISTER "imeRegister"
#define IME_INIT     "imeInit"
#define IME_PREPARE  "imePrepare"
#define IME_FEEDKEY  "imeFeedKey"
#define IME_LISTAREA "imeListArea"
#define IME_KEYAREA  "imeKeyArea"
#define IME_FOCUS    "imeFocus"
#define IME_SIMPLE   "imeSimple"
#define IME_QUIT     "imeQuit"

typedef struct {
  iomeRecord            RECORD;
  iomeInitialize        INIT;
  iomePrepare           PREPARE;
  iomeFeedKey           FEEDKEY;
  iomeMouseOverListArea LISTAREA;
  iomeMouseOverKeyArea  KEYAREA;
  iomeNeedFocus         FOCUS;
  iomeSimpleConvert     SIMPLE;
  iomeQuit              QUIT;
} iomeModule;
/***********************************************\
  Before feedkey, convert to IME KEYCODE
\***********************************************/
#define IME_KEY_F1       0x101
#define IME_KEY_F2       0x102
#define IME_KEY_F3       0x103
#define IME_KEY_F4       0x104
#define IME_KEY_F5       0x105
#define IME_KEY_F6       0x106
#define IME_KEY_F7       0x107
#define IME_KEY_F8       0x108
#define IME_KEY_F9       0x109
#define IME_KEY_F10      0x10A
#define IME_KEY_F11      0x10B
#define IME_KEY_F12      0x10C

#define IME_KEY_HOME     0x201
#define IME_KEY_LEFT     0x202
#define IME_KEY_UP       0x203
#define IME_KEY_RIGHT    0x204
#define IME_KEY_DOWN     0x205
#define IME_KEY_BEGIN    0x206
#define IME_KEY_END      0x207
#define IME_KEY_NEXT     0x208
#define IME_KEY_PAGEUP   0x209
#define IME_KEY_PAGEDOWN 0x20A
#define IME_KEY_INSERT   0x20B
#define IME_KEY_DELETE   0x20C

#define IME_KEY_TAB      0x301
#define IME_KEY_ENTER    0x302
#define IME_KEY_ESC      0x303
#define IME_KEY_BACK     0x304

#define IME_KF_SHIFT     (1<<0)
#define IME_KF_ALT       (1<<1)
#define IME_KF_CONTROL   (1<<2)

#define IME_RT_DUMB      0
#define IME_RT_REJECT    1
#define IME_RT_BLOCKOUT  2
#define IME_RT_SEND      4

#define IME_RT_MASK      0x0000FFFF
#define IME_RT_CRY       0x10000
/***********************************************\
               Default functions
\***********************************************/
int isBig5(unsigned char ch[]);
int isGB2312(unsigned char ch[]);


#ifdef __cplusplus
}
#endif

#endif
